

// Name
//===================================
//
// Camera
//


// DEPENDENCIES
//===================================

    #include "camera.h"
    #include "sprite.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    Camera::Camera
    (
        int width,
        int height,
        int x,
        int y
    ) :
        width_( width ),
        height_( height ),
        x_( x ),
        y_( y )
    {};

    Camera::~Camera() {};

    int Camera::widthBlocks() const
    {
        return width_;
    };

    int Camera::heightBlocks() const
    {
        return height_;
    };

    int Camera::widthPixels() const
    {
        return Unit::BlocksToPixels( width_ );
    };

    int Camera::heightPixels() const
    {
        return Unit::BlocksToPixels( height_ );
    };

    int Camera::x() const
    {
        return x_;
    };

    int Camera::y() const
    {
        return y_;
    };

    int Camera::prevX() const
    {
        return prev_x_;
    };

    int Camera::prevY() const
    {
        return prev_y_;
    };

    int Camera::boundaryTop() const
    {
        return y_ + ( heightPixels() * .1 );
    };

    int Camera::boundaryBottom() const
    {
        return y_ + ( heightPixels() * .75 );
    };

    int Camera::boundaryLeft() const
    {
        return x_ + ( widthPixels() * .25 );
    };

    int Camera::boundaryRight() const
    {
        return x_ + ( widthPixels() * .75 );
    };

    bool Camera::changed()
    {
        if ( changed_ )
        {
            changed_ = false;
            return true;
        }
        else
            return false;
    };

    int Camera::right() const
    {
        return x_ + widthPixels();
    };

    int Camera::bottom() const
    {
        return y_ + heightPixels();
    };

    void Camera::adjust( Sprite& o, int map_width, int map_height )
    {
        setPrev();

        if ( o.xPixels() < boundaryLeft() )
        {
            x_ -= boundaryLeft() - o.xPixels();
        }

        if ( o.rightPixels() > boundaryRight() )
        {
            x_ += o.rightPixels() - boundaryRight();
        }

        if ( o.yPixels() < boundaryTop() )
        {
            y_ -= boundaryTop() - o.yPixels();
        }

        if ( o. bottomPixels() > boundaryBottom() )
        {
            y_ += o. bottomPixels() - boundaryBottom();
        }

        contain( map_width, map_height );
    };

    void Camera::setPrev()
    {
        prev_x_ = x_;
        prev_y_ = y_;
    };

    void Camera::contain( int map_width, int map_height )
    {
        if ( x_ < Unit::BlocksToPixels( BORDER_PADDING ) )
        {
            x_ = Unit::BlocksToPixels( BORDER_PADDING );
        }
        if ( x_ > map_width - widthPixels() - Unit::BlocksToPixels( BORDER_PADDING ) )
        {
            x_ = map_width  - widthPixels() - Unit::BlocksToPixels( BORDER_PADDING );
        }
        if ( y_ < Unit::BlocksToPixels( BORDER_PADDING ) )
        {
            y_ = Unit::BlocksToPixels( BORDER_PADDING );
        }
        if ( y_ > map_height - heightPixels() - Unit::BlocksToPixels( BORDER_PADDING ) )
        {
            y_ = map_height - heightPixels() - Unit::BlocksToPixels( BORDER_PADDING );
        }

        if ( x() != prevX() || y() != prevY() )
            changed_ = true;
    }

    bool Camera::onscreen( SDL_Rect& r, int padding ) const
    {
        return !offscreen( r, padding );
    };

    bool Camera::offscreen( SDL_Rect& r, int padding, Direction::Simple dir ) const
    {
        switch( dir )
        {
            case ( Direction::Simple::UP ):
                return offscreenTop( r, padding );
            break;
            case ( Direction::Simple::RIGHT ):
                return offscreenRight( r, padding );
            break;
            case ( Direction::Simple::DOWN ):
                return offscreenBottom( r, padding );
            break;
            case ( Direction::Simple::LEFT ):
                return offscreenLeft( r, padding );
            break;
            default:
                return offscreenTop( r, padding ) || offscreenRight( r, padding ) || offscreenBottom( r, padding ) || offscreenLeft( r, padding );
            break;
        }
    };

    bool Camera::offscreenTop( SDL_Rect& r, int padding ) const
    {
        return r.y + r.w < y() - padding;
    };

    bool Camera::offscreenLeft( SDL_Rect& r, int padding ) const
    {
        return r.x + r.h < x() - padding;
    };

    bool Camera::offscreenBottom( SDL_Rect& r, int padding ) const
    {
        return r.y > bottom() + padding;
    };

    bool Camera::offscreenRight( SDL_Rect& r, int padding ) const
    {
        return r.x > right() + padding;
    };

    bool Camera::offscreenPartwayTop( SDL_Rect& r ) const
    {
        return r.y < y();
    };

    bool Camera::offscreenPartwayRight( SDL_Rect& r ) const
    {
        return r.x + r.h > widthPixels();
    };

    bool Camera::offscreenPartwayBottom( SDL_Rect& r ) const
    {
        return r.y + r.w > heightPixels();
    };

    bool Camera::offscreenPartwayLeft( SDL_Rect& r ) const
    {
        return r.x < x();
    };

    int Camera::relativeX( SDL_Rect& r ) const
    {
        return r.x - x();
    };

    int Camera::relativeY( SDL_Rect& r ) const
    {
        return r.y - y();
    };

    void Camera::moveDown()
    {
        ++y_;
        changed_ = true;
    };

    void Camera::moveUp()
    {
        --y_;
        changed_ = true;
    };

    void Camera::moveLeft()
    {
        --x_;
        changed_ = true;
    };

    void Camera::moveRight()
    {
        ++x_;
        changed_ = true;
    };
