

// Name
//===================================
//
// Input
//

#ifndef INPUT_H
#define INPUT_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include <map>
    #include <SDL2/SDL.h>


// CLASS
//===================================

    class Input
    {
        public:
            enum Action
            {
                CONFIRM,
                CANCEL,
                MENU,
                JUMP,
                RUN,
                MOVE_UP,
                MOVE_RIGHT,
                MOVE_DOWN,
                MOVE_LEFT,
                EXTRA1,
                EXTRA2
            };

            void update();
            void reset();
            bool pressed( Action action );
            bool released( Action action );
            bool held( Action action );

            void keyPress( SDL_Keycode key );
            void keyRelease( SDL_Keycode key );
            void keyHold( SDL_Keycode key );

        private:
            std::map<Action, SDL_Keycode> buttons_ =
            {
                { Action::CONFIRM   , SDLK_z     },
                { Action::CANCEL    , SDLK_x     },
                { Action::MENU      , SDLK_c     },
                { Action::JUMP      , SDLK_z     },
                { Action::RUN       , SDLK_x     },
                { Action::MOVE_UP   , SDLK_UP    },
                { Action::MOVE_RIGHT, SDLK_RIGHT },
                { Action::MOVE_DOWN , SDLK_DOWN  },
                { Action::MOVE_LEFT , SDLK_LEFT  },
                { Action::EXTRA1    , SDLK_a     },
                { Action::EXTRA2    , SDLK_s     }
            };
            std::map<SDL_Keycode, bool> keys_pressed_;
            std::map<SDL_Keycode, bool> keys_pressed_before_released_;
            std::map<SDL_Keycode, bool> keys_released_;
            std::map<SDL_Keycode, bool> keys_held_;
    };

#endif // INPUT_H

