#include "block_system.h"
#include "camera.h"
#include "event_system.h"
#include "graphics.h"
#include "input.h"
#include "inventory.h"
#include <iostream>
#include "map.h"
#include "player_graphics.h"
#include "player_movement.h"
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include "sprite.h"
#include <vector>

int fpsMilliseconds()
{
    return floor( 1000 / 60 );
};

int main( int argc, char* argv[] )
{

    SDL_Window*  window = nullptr;
    SDL_Surface* screen = nullptr;
    SDL_Renderer* renderer = nullptr;
    SDL_Texture* autumn_texture = nullptr;

    if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        printf( "SDL could not initialize! SDL_Error: %s\n", SDL_GetError() );
        return -1;
    }

    Graphics graphics;

    int img_flags = IMG_INIT_PNG;
    int img_init = IMG_Init( img_flags );
    if( img_init != img_flags )
    {
        printf("IMG_Init: Failed to init required jpg and png support!\n");
        printf("IMG_Init: %s\n", IMG_GetError());
        return -1;
    }

    Input input_;
    Map map_;
    Camera camera_ = Camera();
    EventSystem events_;
    Inventory inventory_;
    BlockSystem blocks_;
    Sprite hero_ = Sprite( new PlayerGraphics(), new PlayerMovement(), 96, 32, 16, 24, Sprite::Type::HERO );
    //Sprite block_ = Sprite( Graphics::SpriteSheet::LVTILESET_CITY, nullptr, 160, 128, 16, 16, Sprite::Type::BLOCK );

    /*
    std::vector<Sprite> blocks_;

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 16, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 32, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 48, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 80, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 96, 32, 16, 16, Sprite::Type::FLAT_TOP, 80, 0 ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 32, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 48, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 64, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 80, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 96, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 112, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 128, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 144, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 160, 16, 16, Sprite::Type::LADDER, 64, 0 ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 176, 16, 16, Sprite::Type::LADDER, 64, 0 ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 16, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 32, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 48, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 80, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 96, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 112, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 128, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 160, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 176, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 192, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 208, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 224, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 176, 16, 16, Sprite::Type::BLOCK ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 16, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 32, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 48, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 80, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 96, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 112, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 128, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 160, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 176, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 192, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 208, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 224, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 160, 16, 16, Sprite::Type::BLOCK ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 112, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 48, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 80, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 80, 16, 16, Sprite::Type::BLOCK ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 16, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 32, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 48, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 64, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 80, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 96, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 112, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 128, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 144, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 160, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 176, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 192, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 208, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 224, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 144, 16, 16, Sprite::Type::BLOCK ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 128, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 240, 112, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 128, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 256, 112, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 272, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 272, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 272, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 288, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 288, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 288, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 272, 176, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 288, 176, 16, 16, Sprite::Type::BLOCK ) );

/*
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 160, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 144, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 128, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 112, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 80, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 64, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 48, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 32, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 16, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 0, 0, 16, 16, Sprite::Type::BLOCK ) );

    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 176, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 192, 96, 16, 16, Sprite::Type::BLOCK ) );
    blocks_.push_back( Sprite( new SpriteGraphics( Graphics::SpriteSheet::LVTILESET_CITY, 80, 0 ), nullptr, 112, 64, 16, 16, Sprite::Type::BLOCK ) );
*/

    bool program_finished = false;
    SDL_Event event;
    int ticks_ = 0;

    while ( !program_finished )
    {
        while ( SDL_PollEvent( &event ) != 0 )
        {
            if ( event.type == SDL_QUIT )
            {
                program_finished = true;
            }
            if ( event.type == SDL_KEYDOWN )
            {
                input_.keyPress( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
                input_.keyHold( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
            }
            if ( event.type == SDL_KEYUP )
            {
                input_.keyRelease( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
            }
            if ( input_.pressed( Input::Action::MENU ) )
            {
                program_finished = true;
            }

        }

            if ( SDL_GetTicks() - ticks_ >= fpsMilliseconds() )
            {

                if ( events_.won_ )
                    program_finished = true;

                blocks_.blocksFromMap( map_, camera_ );
                hero_.update( input_, camera_, map_ );
                //hero_.collisionDetect( block_ );

                graphics.clearScreen();
                graphics.colorCanvas();
                //block_.render( graphics );

                /*
                for ( auto& b : blocks_ )
                {
                    hero_.collisionDetect( b );
                }
                */

                blocks_.collide( hero_, map_, events_, inventory_ );

                camera_.adjust( hero_, map_.widthPixels(), map_.heightPixels() );
                input_.update();

                /*
                for ( auto& b : blocks_ )
                {
                    b.render( graphics, camera_ );
                }
                */

                blocks_.render( graphics, camera_ );
                hero_.render( graphics, camera_ );
                inventory_.render( graphics );

                graphics.presentScreen();

                ticks_ = SDL_GetTicks();
            }
    }

    IMG_Quit();
    SDL_Quit();

    return 0;
}
