

// Name
//===================================
//
// Object
//


// DEPENDENCIES
//===================================

    #include "collision.h"
    #include "object.h"
    #include "unit.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Object::Object
    (
        int x,
        int y,
        int width,
        int height
    )
    :
        physics_bound_box_ ( { Unit::PixelsToSubPixels( x + 1), Unit::PixelsToSubPixels( y + 1), Unit::PixelsToSubPixels( width_ - 2 ), Unit::PixelsToSubPixels( height_ - 1 ) } )
    {};

    Object::~Object() {};

    int Object::heightPixels() const
    {
        return Unit::SubPixelsToPixels( physics_bound_box_.h );
    };

    int Object::xPixels() const
    {
        return Unit::SubPixelsToPixels( physics_bound_box_.x );
    };

    int Object::yPixels() const
    {
        return Unit::SubPixelsToPixels( physics_bound_box_.y );
    };

    int Object::xSubPixels() const
    {
        return physics_bound_box_.x;
    };

    int Object::ySubPixels() const
    {
        return physics_bound_box_.y;
    };

    int Object::hitboxHalfWidth() const
    {
        return round( physics_bound_box_.w / 2 );
    };

    int Object::hitboxHalfHeight() const
    {
        return round( physics_bound_box_.h / 2 );
    };

    int Object::hitboxCenterX() const
    {
        return xSubPixels() + hitboxHalfWidth();
    };

    int Object::hitboxCenterY() const
    {
        return ySubPixels() + hitboxHalfHeight();
    };

    int Object::hitboxLeft() const
    {
        return physics_bound_box_.x;
    };

    int Object::hitboxRight() const
    {
        return physics_bound_box_.x + physics_bound_box_.w;
    };

    int Object::hitboxTop() const
    {
        return physics_bound_box_.y;
    };

    int Object::hitboxBottom() const
    {
        return physics_bound_box_.y + physics_bound_box_.h;
    };

    int Object::hitboxPrevLeft() const
    {
        return xPrevSubPixels();
    };

    int Object::hitboxPrevRight() const
    {
        return xPrevSubPixels() + physics_bound_box_.w;
    };

    int Object::hitboxPrevTop() const
    {
        return yPrevSubPixels();
    };

    int Object::hitboxPrevBottom() const
    {
        return yPrevSubPixels() + physics_bound_box_.h;
    };

    Collision Object::collide( Object* other ) const
    {
        int overlap_x_left   = 0;
        int overlap_x_right  = 0;
        int overlap_y_top    = 0;
        int overlap_y_bottom = 0;

        const int left_bottom_point_x = physics_bound_box_.x + 2000;
        const int right_bottom_point_x = physics_bound_box_.x + physics_bound_box_.w - 2000;
        const int center_bottom_point_x = physics_bound_box_.x + hitboxHalfHeight();
        const int bottom_points_y = physics_bound_box_.y + physics_bound_box_.h;

        const int target_downward_left_bound_x = other->physics_bound_box_.x;
        const int target_downward_right_bound_x = other->physics_bound_box_.x + other->physics_bound_box_.w;
        const int target_downward_top_bound_y = other->physics_bound_box_.y;
        const int target_downward_bottom_bound_y = other->physics_bound_box_.y + other->hitboxHalfHeight();//other->physics_bound_box_.h;

        const int top_points_y = physics_bound_box_.y;
        const int target_upward_top_bound_y = other->physics_bound_box_.y + other->hitboxHalfHeight();
        const int target_upward_bottom_bound_y = other->physics_bound_box_.y + other->physics_bound_box_.h;

        const int left_point_x = physics_bound_box_.x;
        const int left_point_y_top = physics_bound_box_.y;
        const int left_point_y_center = physics_bound_box_.y + hitboxHalfHeight();
        const int left_point_y_bottom = physics_bound_box_.y + physics_bound_box_.h - 8000;

        const int left_target_bound_x_left = other->physics_bound_box_.x;
        const int left_target_bound_x_right = other->physics_bound_box_.x + other->physics_bound_box_.w;
        const int left_target_bound_y_top = other->physics_bound_box_.y;
        const int left_target_bound_y_bottom = other->physics_bound_box_.y + other->physics_bound_box_.h;

        const int right_point_x = physics_bound_box_.x + physics_bound_box_.w;

        const int right_target_bound_x_left = other->physics_bound_box_.x;
        const int right_target_bound_x_right = other->physics_bound_box_.x + other->hitboxHalfWidth();

        if
        (
            ( left_bottom_point_x > target_downward_left_bound_x && left_bottom_point_x < target_downward_right_bound_x ) ||
            ( right_bottom_point_x > target_downward_left_bound_x && right_bottom_point_x < target_downward_right_bound_x ) ||
            ( center_bottom_point_x > target_downward_left_bound_x && center_bottom_point_x < target_downward_right_bound_x )
        )
        {
            if
            (
                bottom_points_y > target_downward_top_bound_y &&
                bottom_points_y < target_downward_bottom_bound_y
            )
            {
                overlap_y_bottom = bottom_points_y - target_downward_top_bound_y;
            }
        }

        if
        (
            ( left_bottom_point_x > target_downward_left_bound_x && left_bottom_point_x < target_downward_right_bound_x ) ||
            ( right_bottom_point_x > target_downward_left_bound_x && right_bottom_point_x < target_downward_right_bound_x ) ||
            ( center_bottom_point_x > target_downward_left_bound_x && center_bottom_point_x < target_downward_right_bound_x )
        )
        {
            if
            (
                top_points_y < target_upward_bottom_bound_y &&
                top_points_y > target_upward_top_bound_y
            )
            {
                overlap_y_top = target_upward_bottom_bound_y - top_points_y;
            }
        }

        if
        (
            ( left_point_y_top > left_target_bound_y_top && left_point_y_top < left_target_bound_y_bottom ) ||
            ( left_point_y_bottom > left_target_bound_y_top && left_point_y_bottom < left_target_bound_y_bottom ) ||
            ( left_point_y_center > left_target_bound_y_top && left_point_y_center < left_target_bound_y_bottom )
        )
        {
            if
            (
                left_point_x > left_target_bound_x_left &&
                left_point_x < left_target_bound_x_right
            )
            {
                overlap_x_left = left_target_bound_x_right - left_point_x;
            }
        }

        if
        (
            ( left_point_y_top > left_target_bound_y_top && left_point_y_top < left_target_bound_y_bottom ) ||
            ( left_point_y_bottom > left_target_bound_y_top && left_point_y_bottom < left_target_bound_y_bottom ) ||
            ( left_point_y_center > left_target_bound_y_top && left_point_y_center < left_target_bound_y_bottom )
        )
        {
            if
            (
                right_point_x > right_target_bound_x_left &&
                right_point_x < right_target_bound_x_right
            )
            {
                overlap_x_right = right_point_x - right_target_bound_x_left;
            }
        }

        return Collision( overlap_x_left, overlap_x_right, overlap_y_top, overlap_y_bottom );
    };

    int Object::xPrevSubPixels() const
    {
        return x_prev_;
    };

    int Object::yPrevSubPixels() const
    {
        return y_prev_;
    };

    int Object::xPrevPixels() const
    {
        return Unit::SubPixelsToPixels( x_prev_ );
    };

    int Object::yPrevPixels() const
    {
        return Unit::SubPixelsToPixels( y_prev_ );
    };

    int Object::rightPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxRight() );
    };

    int Object::bottomPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxBottom() );
    };

    int Object::rightSubPixels() const
    {
        return hitboxRight();
    };

    int Object::bottomSubPixels() const
    {
        return hitboxBottom();
    };

    int Object::rightPrevPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxPrevRight() );
    };

    int Object::bottomPrevPixels() const
    {
        return Unit::SubPixelsToPixels( hitboxPrevBottom() );
    };

    int Object::rightPrevSubPixels() const
    {
        return hitboxPrevRight();
    };

    int Object::bottomPrevSubPixels() const
    {
        return hitboxPrevBottom();
    };
