
// Name
//===================================
//
// Object
//

#ifndef OBJECT_H
#define OBJECT_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Collision;
    class Graphics;
    class Input;


// DEPENDENCIES
//===================================

    #include "direction.h"
    #include <SDL2/SDL.h>
    #include "unit.h"


// CLASS
//===================================

    class Object
    {
        public:
            Object
            (
                int x = 0,
                int y = 0,
                int width = 16,
                int height = 16
            );
            ~Object();

            int width_ = 16;
            int height_ = 24;

            SDL_Rect physics_bound_box_ =
            {
                Unit::PixelsToSubPixels( Unit::WINDOW_WIDTH_PIXELS / 2 ),
                0,
                Unit::PixelsToSubPixels( width_ - 2 ),
                Unit::PixelsToSubPixels( height_ - 2 )
            };

            virtual void render( Graphics& graphics, Camera& camera ) = 0;
            Collision collide( Object* other ) const;

            int xPixels() const;
            int yPixels() const;
            int xSubPixels() const;
            int ySubPixels() const;
            int xPrevSubPixels() const;
            int yPrevSubPixels() const;
            int xPrevPixels() const;
            int yPrevPixels() const;
            int heightPixels() const;
            int hitboxHalfWidth() const;
            int hitboxHalfHeight() const;
            int hitboxCenterX() const;
            int hitboxCenterY() const;
            int hitboxLeft() const;
            int hitboxRight() const;
            int hitboxTop() const;
            int hitboxBottom() const;
            int hitboxPrevLeft() const;
            int hitboxPrevRight() const;
            int hitboxPrevTop() const;
            int hitboxPrevBottom() const;
            int rightPixels() const;
            int bottomPixels() const;
            int rightSubPixels() const;
            int bottomSubPixels() const;
            int rightPrevPixels() const;
            int bottomPrevPixels() const;
            int rightPrevSubPixels() const;
            int bottomPrevSubPixels() const;

        protected:
            int x_prev_ = 0;
            int y_prev_ = 0;
    };

#endif // OBJECT_H

